import pygame

# By Willi Kappler <grandor@gmx.de>
# Licensed under GPL

class LevelEditor:
    def __init__(self, screen, smfont, level):
        self.screen = screen
        self.black = (0, 0, 0)
        self.font = smfont
        self.level = level
        self.layer = 0
        self.leftTool = [1,1]
        self.rightTool = [0,0]
        self.snd = pygame.mixer.Sound("snd/beeps2.ogg")
        self.maxTool = [len(self.level.backGfx), len(self.level.frontGfx)]

    def displayInfo(self):
        self.font.write(490, 10, "LEVEL: " + str(self.level.level))
        if self.layer == 0:
            self.font.write(490, 40, "LAYER: 1")
        elif self.layer == 1:
            self.font.write(490, 40, "LAYER: 2")
        elif self.layer == 2:
            self.font.write(490, 40, "LAYER: 1 2")
        elif self.layer == 3:
            self.font.write(490, 40, "LAYER: 2 1")
        self.font.write(490, 100, "CHNG TOOL:")
        self.font.write(490, 120, "+ - AND")
        self.font.write(490, 140, "SHIFT + -")
        self.font.write(490, 160, "SHIFT LMB")
        self.font.write(490, 180, "SHIFT RMB")

        self.font.write(490, 220, "N P: LEVEL")
        self.font.write(490, 240, "C: CLEAR")
        self.font.write(490, 260, "S: SAVE")
        self.font.write(490, 280, "ESC: BACK")
        self.font.write(490, 320, "EDIT WITH:")
        self.font.write(490, 340, "LMB + RMB")

        self.font.write(490, 380, "1 2 AND")
        self.font.write(490, 400, "3 4: LAYER")
        pygame.display.flip()

    def displayTools(self):
        if self.layer == 0 or self.layer == 2:
            self.screen.blit(self.level.backGfx[self.leftTool[0]], (576,448))
            self.screen.blit(self.level.backGfx[self.rightTool[0]], (608,448))
        else:
            self.screen.blit(self.level.backGfx[0], (576,448))
            self.screen.blit(self.level.backGfx[0], (608,448))

            t = self.leftTool[1]
            if t > 0:
                self.screen.blit(self.level.frontGfx[t], (576,448))

            t = self.rightTool[1]
            if t > 0:
                self.screen.blit(self.level.frontGfx[t], (608,448))

        pygame.display.flip()

    def refresh(self):
        self.screen.fill(self.black)
        self.level.display(self.layer)
        self.displayInfo()
        self.displayTools()

    def incLeftTool(self):
        if self.layer == 0 or self.layer == 2:
            self.leftTool[0] += 1
            if self.leftTool[0] == self.maxTool[0]:
                self.leftTool[0] = 0
        else:
            self.leftTool[1] += 1
            if self.leftTool[1] == self.maxTool[1]:
                self.leftTool[1] = 0

        self.displayTools()

    def incRightTool(self):
        if self.layer == 0 or self.layer == 2:
            self.rightTool[0] += 1
            if self.rightTool[0] == self.maxTool[0]:
                self.rightTool[0] = 0
        else:
            self.rightTool[1] += 1
            if self.rightTool[1] == self.maxTool[1]:
                self.rightTool[1] = 0

        self.displayTools()

    def decLeftTool(self):
        if self.layer == 0 or self.layer == 2:
            self.leftTool[0] -= 1
            if self.leftTool[0] == -1:
                self.leftTool[0] = self.maxTool[0] - 1
        else:
            self.leftTool[1] -= 1
            if self.leftTool[1] == -1:
                self.leftTool[1] = self.maxTool[1] - 1

        self.displayTools()

    def decRightTool(self):
        if self.layer == 0 or self.layer == 2:
            self.rightTool[0] -= 1
            if self.rightTool[0] == -1:
                self.rightTool[0] = self.maxTool[0] - 1
        else:
            self.rightTool[1] -= 1
            if self.rightTool[1] == -1:
                self.rightTool[1] = self.maxTool[1] - 1

        self.displayTools()

    def run(self):
        self.layer = 0
        pygame.mouse.set_visible(1)
        self.level.level = 1
        self.level.load()
        self.refresh()

        quitEditor = False
        while not quitEditor:
            event = pygame.event.wait()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    quitEditor = True
                elif event.key == pygame.K_PLUS:
                    keys = pygame.key.get_mods()
                    if ((keys & pygame.KMOD_LSHIFT) == pygame.KMOD_LSHIFT) or ((keys & pygame.KMOD_RSHIFT) == pygame.KMOD_RSHIFT):
                        self.incRightTool()
                    else:
                        self.incLeftTool()
                elif event.key == pygame.K_MINUS:
                    keys = pygame.key.get_mods()
                    if ((keys & pygame.KMOD_LSHIFT) == pygame.KMOD_LSHIFT) or ((keys & pygame.KMOD_RSHIFT) == pygame.KMOD_RSHIFT):
                        self.decRightTool()
                    else:
                        self.decLeftTool()
                elif event.key == pygame.K_s:
                    self.level.save()
                    self.snd.play()
                elif event.key == pygame.K_c:
                    if self.layer == 0 or self.layer == 2:
                        self.level.clear(self.layer, self.rightTool[0])
                        self.level.display(self.layer)
                    else:
                        self.level.clear(self.layer, self.rightTool[1])
                        self.level.display(self.layer)
                    pygame.display.flip()
                elif event.key == pygame.K_n:
                    self.level.next()
                    self.refresh()
                elif event.key == pygame.K_p:
                    self.level.prev()
                    self.refresh()
                elif event.key == pygame.K_1:
                    self.layer = 0
                    self.refresh()
                elif event.key == pygame.K_2:
                    self.layer = 1
                    self.refresh()
                elif event.key == pygame.K_3:
                    self.layer = 2
                    self.refresh()
                elif event.key == pygame.K_4:
                    self.layer = 3
                    self.refresh()
            elif (event.type == pygame.MOUSEBUTTONDOWN) or (event.type == pygame.MOUSEMOTION):
                (mx,my) = pygame.mouse.get_pos()
                (lmb,mmb,rmb) = pygame.mouse.get_pressed()
                if mx < 480:
                    mx = mx / 32
                    my = my / 32
                    keys = pygame.key.get_pressed()
                    if keys[pygame.K_LSHIFT] or keys[pygame.K_RSHIFT]:
                        if lmb:
                            if self.layer == 0 or self.layer == 2:
                                self.leftTool[0] = self.level.backData[my][mx]
                            else:
                                self.leftTool[1] = self.level.frontData[my][mx]
                            self.displayTools()
                        elif rmb:
                            if self.layer == 0 or self.layer == 2:
                                self.rightTool[0] = self.level.backData[my][mx]
                            else:
                                self.rightTool[1] = self.level.frontData[my][mx]
                            self.displayTools()
                    else:
                        if lmb:
                            if self.layer == 0 or self.layer == 2:
                                self.level.set(self.layer, mx, my, self.leftTool[0])
                            else:
                                self.level.set(self.layer, mx, my, self.leftTool[1])
                            pygame.display.flip()

                        elif rmb:
                            if self.layer == 0 or self.layer == 2:
                                self.level.set(self.layer, mx, my, self.rightTool[0])
                            else:
                                self.level.set(self.layer, mx, my, self.rightTool[1])
                            pygame.display.flip()

