import pygame

# By Willi Kappler <grandor@gmx.de>
# Licensed under GPL

class Object:
    def __init__(self, screen, level, gfx, x, y):
        self.screen = screen
        self.level = level
        self.gfxList = gfx
        self.black = (0,0,0)
        self.x = x
        self.y = y
        self.animList = []
        self.animPtr = 0
        self.animFrame = 0
        self.animFrameMax = 0
        self.animTimes = [20,20,20,20]
        self.timeElapsed = 0
        self.visible = True
        self.direction = 0
        self.speed = 2
        self.fallSpeed = 4
        self.normalSpeed = 2
        self.moveAction = [self.stop, self.stop, self.stop, self.stop]
        self.moveAnim = [0,0,0,0,0]
        self.objType = 0
        self.asleep = False
        self.sleepTime = 0

    def moveRight(self):
        if self.direction != 1:
            self.speed = self.normalSpeed
            self.direction = 1
            self.animPtr = self.moveAnim[0]
            self.animFrame = 0
            self.animFrameMax = len(self.animList[self.animPtr]) - 1

    def moveLeft(self):
        if self.direction != 2:
            self.speed = self.normalSpeed
            self.direction = 2
            self.animPtr = self.moveAnim[1]
            self.animFrame = 0
            self.animFrameMax = len(self.animList[self.animPtr]) - 1

    def moveUp(self):
        if self.direction != 3:
            self.speed = self.normalSpeed
            self.direction = 3
            self.animPtr = self.moveAnim[2]
            self.animFrame = 0
            self.animFrameMax = len(self.animList[self.animPtr]) - 1

    def moveDown(self):
        if self.direction != 4:
            self.speed = self.fallSpeed
            self.direction = 4
            self.animPtr = self.moveAnim[3]
            self.animFrame = 0
            self.animFrameMax = len(self.animList[self.animPtr]) - 1

    def stop(self):
        if self.direction != 0:
            self.direction = 0
            self.animPtr = self.moveAnim[4]
            self.animFrame = 0
            self.animFrameMax = len(self.animList[self.animPtr]) - 1
            self.y = (self.y / 32) * 32

    def move(self):
        if self.direction == 1: # right
            if self.x < 448 - self.speed:
                self.x += self.speed
            else:
                self.x = 448
                self.moveAction[0]()
        elif self.direction == 2: # left
            if self.x > self.speed:
                self.x -= self.speed
            else:
                self.x = 0
                self.moveAction[1]()
        if self.direction == 3: # up
            if self.y > self.speed:
                self.y -= self.speed
            else:
                self.y = 0
                self.moveAction[2]()
        elif self.direction == 4: # down
            if self.y < 448 - self.speed:
                self.y += self.speed
            else:
                self.y = 448
                self.moveAction[3]()

    def check(self): # implement in your subclass
        pass

    def getLT(self, dx = 0): # out left top hotspot
        x = (self.x + dx - 1) / 32
        y = self.y / 32
        if x >= 0:
            return self.level.frontData[y][x]
        return -1

    def getLB(self, dx = 0): # out left bottom hotspot
        x = (self.x + dx - 1) / 32
        y = (self.y + 31) / 32
        if x >= 0:
            return self.level.frontData[y][x]
        return -1

    def getLLT(self, dx = 0): # out left left top hotspot
        x = (self.x + dx - 33) / 32
        y = self.y / 32
        if x >= 0:
            return self.level.frontData[y][x]
        return -1

    def getLLB(self, dx = 0): # out left left bottom hotspot
        x = (self.x + dx - 33) / 32
        y = (self.y + 31) / 32
        if x >= 0:
            return self.level.frontData[y][x]
        return -1

    def getRT(self, dx = 0): # out right top hotspot
        x = (self.x - dx + 32) / 32
        y = self.y / 32
        if x <= 14:
            return self.level.frontData[y][x]
        return -1

    def getRB(self, dx = 0): # out right bottom hotspot
        x = (self.x - dx + 32) / 32
        y = (self.y + 31) / 32
        if x <= 14:
            return self.level.frontData[y][x]
        return -1

    def getRRT(self, dx = 0): # out right right top hotspot
        x = (self.x - dx + 64) / 32
        y = self.y / 32
        if x <= 14:
            return self.level.frontData[y][x]
        return -1

    def getRRB(self, dx = 0): # out right right bottom hotspot
        x = (self.x - dx + 64) / 32
        y = (self.y + 31) / 32
        if x <= 14:
            return self.level.frontData[y][x]
        return -1

    def getTL(self, dx = 0): # out top left hotspot
        x = (self.x + dx) / 32
        y = (self.y - 1) / 32
        if y >= 0:
            return self.level.frontData[y][x]
        return -1

    def getTR(self, dx = 0): # out top right hotspot
        x = (self.x - dx + 31) / 32
        y = (self.y - 1) / 32
        if y >= 0:
            return self.level.frontData[y][x]
        return -1

    def getBL(self, dx = 0): # out bottom left hotspot
        x = (self.x + dx) / 32
        y = (self.y + 32) / 32
        if y <= 14:
            return self.level.frontData[y][x]
        return -1

    def getBR(self, dx = 0): # out bottom right hotspot
        x = (self.x - dx + 31) / 32
        y = (self.y + 32) / 32
        if y <= 14:
            return self.level.frontData[y][x]
        return -1


    def checkDown(self, dx = 0):
        bl = self.getBL(dx)
        br = self.getBR(dx)
        if ((bl == 0) or (bl >= self.level.keyNo)) and ((br == 0) or (br >= self.level.keyNo)):
            return True
        return False

    def checkRight(self, dx = 0):
        rt = self.getRT(dx)
        rb = self.getRB(dx)
        if ((rt > 0) or (rb > 0)) and ((rt < self.level.doorOpened) or (rb < self.level.doorOpened)):
            return True
        return False

    def checkLeft(self, dx = 0):
        lt = self.getLT(dx)
        lb = self.getLB(dx)
        if ((lt > 0) or (lb > 0)) and ((lt < self.level.doorOpened) or (lb < self.level.doorOpened)):
            return True
        return False

    def animate(self):
        t = pygame.time.get_ticks()
        if t > self.timeElapsed + self.animTimes[self.animPtr]:
            self.timeElapsed = t
            self.animFrame += 1
            if self.animFrame > self.animFrameMax:
                self.animFrame = 0

    def sleep(self):
        self.asleep = True
        self.sleepTime = pygame.time.get_ticks()

    def display(self):
        self.screen.blit(self.gfxList[self.animList[self.animPtr][self.animFrame]], (self.x, self.y))


    def go(self):
        if self.visible:
            self.check()
            self.move()
            self.animate()
            self.display()

    def restoreBG(self):
        x1 = self.x / 32
        x2 = (self.x + 31) / 32
        y1 = self.y / 32
        y2 = (self.y + 31) / 32
        if x1 == x2:
            if (y1 == y2) or (y2 == 15):
                self.level.displayXY(x1, y1)
            else:
                self.level.displayXY(x1, y1)
                self.level.displayXY(x1, y2)
        else:
            if (y1 == y2):
                if x2 == 15:
                    self.level.displayXY(x1, y1)
                else:
                    self.level.displayXY(x1, y1)
                    self.level.displayXY(x2, y1)
            else:
                self.level.displayXY(x1, y1)
                self.level.displayXY(x1, y2)
                self.level.displayXY(x2, y1)
                self.level.displayXY(x2, y2)

