import pygame
import playerobj
import snowballobj
import skullobj
import ghostobj
import shredderobj

# By Willi Kappler <grandor@gmx.de>
# Licensed under GPL

class Level:
    def __init__(self, screen, font, options):
        self.screen = screen
        self.black = (0,0,0)
        self.level = 1
        self.font = font
        self.options = options
        self.maxLevel = 51
        self.backData = []
        self.frontData = []
        self.playerX = 0
        self.playerY = 0
        self.snowballX = 1
        self.snowballY = 0
        self.quitGame = False
        self.enemies = []
        self.transporters = []
        self.clock = pygame.time.Clock()

        self.backGfx = []
        self.loadGfx(self.backGfx, "backgrounds.png", 4, 8)

        self.frontGfx = [None]
        self.loadGfx(self.frontGfx, "walls.png", 4, 8)

        self.objNo = len(self.frontGfx)
        self.loadGfx(self.frontGfx, "objects.png", 5, 8)
        self.boxNo = self.objNo + 1
        self.boxGfx = [self.frontGfx[self.boxNo]]
        self.spikeNo = self.boxNo + 1
        self.redOn = self.objNo + 5
        self.yellowOn = self.redOn + 1
        self.greenOn = self.yellowOn + 1
        self.blueOn = self.greenOn + 1
        self.switch1R = self.blueOn + 1
        self.switch2R = self.switch1R + 1
        self.switch3R = self.switch2R + 1
        self.switch4R = self.switch3R + 1
        self.switch1L = self.switch4R + 1
        self.switch2L = self.switch1L + 1
        self.switch3L = self.switch2L + 1
        self.switch4L = self.switch3L + 1
        self.transporterNo = self.switch4L + 1
        self.doorClosed = self.transporterNo + 1
        self.doorOpened = self.doorClosed + 1
        self.redOff = self.doorOpened + 1
        self.yellowOff = self.redOff + 1
        self.greenOff = self.yellowOff + 1
        self.blueOff = self.greenOff + 1
        self.keyNo = self.blueOff + 1
        self.heartNo = self.keyNo + 1
        self.coin1 = self.heartNo + 1
        self.coin2 = self.coin1 + 1
        self.sapphire1 = self.coin2 + 1
        self.sapphire2 = self.sapphire1 + 1
        self.sapphire3 = self.sapphire2 + 1
        self.sapphire4 = self.sapphire3 + 1
        self.sapphire5 = self.sapphire4 + 1
        self.sapphire6 = self.sapphire5 + 1
        self.clockNo = self.sapphire6 + 1
        self.ladder1 = self.clockNo + 1
        self.ladder2 = self.ladder1 + 1
        self.trapNo = self.ladder2 + 1

        self.playerNo = len(self.frontGfx)
        self.loadGfx(self.frontGfx, "player.png", 1, 1)

        self.snowballNo = self.playerNo + 1
        self.loadGfx(self.frontGfx, "snowball.png", 1, 1)

        self.skullNo = self.snowballNo + 1
        self.loadGfx(self.frontGfx, "skull.png", 1, 1)

        self.ghostNo = self.skullNo + 1
        self.loadGfx(self.frontGfx, "ghost.png", 1, 1)

        self.shredderNo = self.ghostNo + 1
        self.loadGfx(self.frontGfx, "shredder.png", 1, 1)

        self.enemies = []
        self.playerGfx = []
        self.loadGfx(self.playerGfx, "player.png", 1, 9)
        self.snowballGfx = []
        self.loadGfx(self.snowballGfx, "snowball.png", 1, 8)
        self.skullGfx = []
        self.loadGfx(self.skullGfx, "skull.png", 1, 8)
        self.ghostGfx = []
        self.loadGfx(self.ghostGfx, "ghost.png", 1,4)
        self.shredderGfx = []
        self.loadGfx(self.shredderGfx, "shredder.png", 1, 10)

        self.gameOverGfx = pygame.image.load("gfx/gameover.png")

        self.playerDieSnd = pygame.mixer.Sound("snd/player_die.ogg")
        self.exitSnd = pygame.mixer.Sound("snd/exit.ogg")
        self.laserSnd = pygame.mixer.Sound("snd/laser.ogg")
        self.transporterSnd = pygame.mixer.Sound("snd/transporter.ogg")
        self.iceblockSnd = pygame.mixer.Sound("snd/iceblock.ogg")
        self.fillSnd = self.iceblockSnd
#        self.fillSnd = pygame.mixer.Sound("snd/fancy_fill.ogg")
        self.keySnd = pygame.mixer.Sound("snd/key.ogg")
        self.doorSnd = pygame.mixer.Sound("snd/door.ogg")
        self.jumpSnd = pygame.mixer.Sound("snd/jump.ogg")
        self.heartSnd = pygame.mixer.Sound("snd/heart.ogg")
        self.boxSnd = pygame.mixer.Sound("snd/box.ogg")
        self.pointSnd = pygame.mixer.Sound("snd/point.ogg")
        self.noBlocksSnd = pygame.mixer.Sound("snd/tick.ogg")
        self.clockSnd = pygame.mixer.Sound("snd/clock.ogg")

    def loadGfx(self, gfxList, name, row, col):
        lgfx = pygame.image.load("gfx/" + name)
        y = 0
        for i in range(row):
            x = 0
            for j in range(col):
                gfx = pygame.Surface((32,32), pygame.SWSURFACE, lgfx)
                gfx.blit(lgfx, (0,0), (x,y,32,32))
                gfxList.append(gfx)
                x += 33
            y += 33

    def load(self):
        self.backData = [[0 for j in range(15)] for i in range(15)]
        self.frontData = [[0 for j in range(15)] for i in range(15)]
        self.playerX = 0
        self.playerY = 0
        self.snowballX = 1
        self.snowballY = 0
        self.frontData[0][0] = self.playerNo
        self.frontData[0][1] = self.snowballNo

        filename = "levels/level" + str(self.level) + ".txt"
        try:
            handle = file(filename, "rt")
        except IOError:
            print "could not load file: ", filename
            return

        for y in range(15):
            line = handle.readline().split()
            for x in range(15):
                t = int(line[x])
                self.backData[y][x] = t

        handle.readline()

        for y in range(15):
            line = handle.readline().split()
            for x in range(15):
                t = int(line[x])
                self.frontData[y][x] = t
                if t == self.playerNo:
                    self.playerX = x
                    self.playerY = y
                elif t == self.snowballNo:
                    self.snowballX = x
                    self.snowballY = y

        handle.close()

    def save(self):
        filename = "levels/level" + str(self.level) + ".txt"
        try:
            handle = file(filename, "wt")
        except IOError:
            print "could not save file: ", filename
            return
        for y in range(15):
            for x in range(15):
                handle.write(str(self.backData[y][x]) + " ")
            handle.write("\n")

        handle.write("\n")

        for y in range(15):
            for x in range(15):
                handle.write(str(self.frontData[y][x]) + " ")
            handle.write("\n")

        handle.close()

    def clear(self, layer, t):
        if layer == 0 or layer == 2: # change background only
            for y in range(15):
                for x in range(15):
                    self.backData[y][x] = t
        else: # change foreground only
            for y in range(15):
                for x in range(15):
                    self.frontData[y][x] = t
            self.playerX = 0
            self.playerY = 0
            self.snowballX = 1
            self.snowballY = 0
            self.frontData[0][0] = self.playerNo
            self.frontData[0][1] = self.snowballNo

    def next(self):
        self.level += 1
        if self.level == self.maxLevel:
            self.level = 1
        self.load()

    def prev(self):
        self.level -= 1
        if self.level == 0:
            self.level = self.maxLevel - 1
        self.load()

    def set(self, layer, x, y, t):
        if layer == 0: # change background, background visible
            self.backData[y][x] = t
            self.screen.blit(self.backGfx[t], (x*32,y*32))
        elif layer == 1: # change foreground, foreground visible
            self.frontData[y][x] = t
            if t == 0:
                self.screen.blit(self.backGfx[0], (x*32,y*32))
            elif t == self.playerNo:
                if (self.playerX != x) or (self.playerY != y):
                    self.frontData[self.playerY][self.playerX] = 0
                    self.screen.blit(self.backGfx[0], (self.playerX*32,self.playerY*32))
                    self.screen.blit(self.backGfx[0], (x*32,y*32))
                    self.screen.blit(self.frontGfx[t], (x*32,y*32))
                    self.playerX = x
                    self.playerY = y
            elif t == self.snowballNo:
                if (self.snowballX != x) or (self.snowballY != y):
                    self.frontData[self.snowballY][self.snowballX] = 0
                    self.screen.blit(self.backGfx[0], (self.snowballX*32,self.snowballY*32))
                    self.screen.blit(self.backGfx[0], (x*32,y*32))
                    self.screen.blit(self.frontGfx[t], (x*32,y*32))
                    self.snowballX = x
                    self.snowballY = y
            else:
                self.screen.blit(self.backGfx[0], (x*32,y*32))
                self.screen.blit(self.frontGfx[t], (x*32,y*32))
        elif layer == 2: # change background, both visible
            self.backData[y][x] = t
            u = self.frontData[y][x]
            if u == 0:
                self.screen.blit(self.backGfx[t], (x*32,y*32))
            else:
                self.screen.blit(self.backGfx[t], (x*32,y*32))
                self.screen.blit(self.frontGfx[u], (x*32,y*32))
        elif layer == 3: # change foreground, both visible
            self.frontData[y][x] = t
            if t == 0:
                self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
            elif t == self.playerNo:
                if (self.playerX != x) or (self.playerY != y):
                    self.frontData[self.playerY][self.playerX] = 0
                    self.screen.blit(self.backGfx[self.backData[self.playerY][self.playerX]], (self.playerX*32,self.playerY*32))
                    self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
                    self.screen.blit(self.frontGfx[t], (x*32,y*32))
                    self.playerX = x
                    self.playerY = y
            elif t == self.snowballNo:
                if (self.snowballX != x) or (self.snowballY != y):
                    self.frontData[self.snowballY][self.snowballX] = 0
                    self.screen.blit(self.backGfx[self.backData[self.snowballY][self.snowballX]], (self.snowballX*32,self.snowballY*32))
                    self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
                    self.screen.blit(self.frontGfx[t], (x*32,y*32))
                    self.snowballX = x
                    self.snowballY = y
            else:
                self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
                self.screen.blit(self.frontGfx[t], (x*32,y*32))

    def display(self, layer):
        if layer == 0:
            for y in range(15):
                for x in range(15):
                    self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
        elif layer == 1:
            for y in range(15):
                for x in range(15):
                    t = self.frontData[y][x]
                    if t == 0:
                        self.screen.blit(self.backGfx[0], (x*32,y*32))
                    else:
                        self.screen.blit(self.frontGfx[t], (x*32,y*32))

        elif layer == 2 or layer == 3:
            for y in range(15):
                for x in range(15):
                    t = self.frontData[y][x]
                    if t == 0:
                        self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
                    else:
                        self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
                        self.screen.blit(self.frontGfx[t], (x*32,y*32))

    def displayXY(self, x, y):
        self.screen.blit(self.backGfx[self.backData[y][x]], (x*32,y*32))
        t = self.frontData[y][x]
        if t > 0:
            self.screen.blit(self.frontGfx[t], (x*32,y*32))

    def go(self):
        for e in self.enemies:
            e.restoreBG()
        self.snowball.restoreBG()
        self.player.restoreBG()

        for e in self.enemies:
            e.go()
        self.snowball.go()
        self.player.go()

    def doSwitch1(self):
        self.doSwitch(self.redOn, self.redOff, self.yellowOn, self.yellowOff)

    def doSwitch2(self):
        self.doSwitch(self.yellowOn, self.yellowOff, self.greenOn, self.greenOff)

    def doSwitch3(self):
        self.doSwitch(self.greenOn, self.greenOff, self.blueOn, self.blueOff)

    def doSwitch4(self):
        self.doSwitch(self.blueOn, self.blueOff, self.redOn, self.redOff)

    def doSwitch(self, on1, off1, on2, off2):
        for y in range(15):
            for x in range(15):
                t = self.frontData[y][x]
                if t == on1:
                    self.frontData[y][x] = off1
                    self.displayXY(x,y)
                    self.laserSnd.play()
                elif t == off1:
                    self.frontData[y][x] = on1
                    self.displayXY(x,y)
                    self.laserSnd.play()
                elif t == on2:
                    self.frontData[y][x] = off2
                    self.displayXY(x,y)
                    self.laserSnd.play()
                elif t == off2:
                    self.frontData[y][x] = on2
                    self.displayXY(x,y)
                    self.laserSnd.play()

    def checkObj(self):
        self.transporters = []
        for y in range(15):
            for x in range(15):
                if self.frontData[y][x] == 1:
                    if self.player.blocks > 0:
                        self.player.blocks -= 1
                if self.frontData[y][x] == self.transporterNo:
                    self.transporters.append((x,y - 1))

    def createEnemies(self):
        self.enemies = []
        for y in range(15):
            for x in range(15):
                t = self.frontData[y][x]
                if t == self.skullNo:
                    self.enemies.append(skullobj.Skull(self.screen, self, self.skullGfx, x*32, y*32))
                    self.frontData[y][x] = 0
                elif t == self.ghostNo:
                    self.enemies.append(ghostobj.Ghost(self.screen, self, self.ghostGfx, x*32, y*32))
                    self.frontData[y][x] = 0
                elif t == self.shredderNo:
                    self.enemies.append(shredderobj.Shredder(self.screen, self, self.shredderGfx, x*32, y*32))
                    self.frontData[y][x] = 0

    def fancyFill(self, mode):
        x = 7
        y = 7
        dx = 1
        dy = 1
        move = 0
        direction = 0
        displayCounter = 0
        self.fillSnd.play()
        while True:
            if mode == 0:
                self.displayXY(x,y)
            else:
                self.screen.blit(self.frontGfx[mode], (x*32,y*32))

            if (x == 0) and (y == 0):
                break

            if direction == 0:
                y -= 1
                move += 1
                if move == dy:
                    direction = 1
                    dy += 1
                    move = 0
            elif direction == 1:
                x += 1
                move += 1
                if move == dx:
                    direction = 2
                    dx += 1
                    move = 0
            elif direction == 2:
                y += 1
                move += 1
                if move == dy:
                    direction = 3
                    dy += 1
                    move = 0
            elif direction == 3:
                x -= 1
                move += 1
                if move == dx:
                    direction = 0
                    dx += 1
                    move = 0

            displayCounter += 1
            if displayCounter == 6:
                self.clock.tick(30)
                pygame.display.flip()
                displayCounter = 0

    def initDisplay(self):
        self.frontData[self.playerY][self.playerX] = 0
        self.frontData[self.snowballY][self.snowballX] = 0
        self.fancyFill(2)
        self.screen.fill(self.black, (480,0,160,480))
        self.fancyFill(0)
        self.font.write(490, 10, "LEVEL:" + str(self.level))
        self.font.write(490, 30, "LIVES:" + str(self.player.lives))
        self.font.write(490, 50, "SCORE:" + str(self.player.points))
        self.font.write(490, 110, "BLOCKS:")
        self.player.displayBlocks()

    def createObj(self):
        self.createEnemies()
        self.player = playerobj.Player(self.screen, self, self.playerGfx, self.playerX*32, self.playerY*32)
        self.snowball = snowballobj.Snowball(self.screen, self, self.snowballGfx, self.snowballX*32, self.snowballY*32)
        self.checkObj()
        self.initDisplay()

    def setSoundVol(self):
        self.playerDieSnd.set_volume(self.options.soundVol)
        self.exitSnd.set_volume(self.options.soundVol)
        self.laserSnd.set_volume(self.options.soundVol)
        self.transporterSnd.set_volume(self.options.soundVol)
        self.iceblockSnd.set_volume(self.options.soundVol)
        self.fillSnd.set_volume(self.options.soundVol)
        self.keySnd.set_volume(self.options.soundVol)
        self.doorSnd.set_volume(self.options.soundVol)
        self.jumpSnd.set_volume(self.options.soundVol)
        self.heartSnd.set_volume(self.options.soundVol)
        self.boxSnd.set_volume(self.options.soundVol)
        self.pointSnd.set_volume(self.options.soundVol)
        self.noBlocksSnd.set_volume(self.options.soundVol)

    def setMusicVol(self):
        pass

    def restart(self):
        self.createEnemies()
        self.player.x = self.playerX * 32
        self.player.y = self.playerY * 32
        self.snowball.x = self.snowballX * 32
        self.snowball.y = self.snowballY * 32
        self.player.blocksMax = 12
        self.player.blocks = self.player.blocksMax
        self.checkObj()
        self.initDisplay()

    def exitReached(self):
        self.player.points += 10
        self.exitSnd.play()
        self.next()
        self.restart()
