import pygame
import gfxobject
import random

# By Willi Kappler <grandor@gmx.de>
# Licensed under GPL

class Player(gfxobject.Object):
    def __init__(self, screen, level, gfx, x, y):
        gfxobject.Object.__init__(self, screen, level, gfx, x, y)

        self.animList = [[0,1],[3,4],[6,7,8,7],[2],[5]]
        self.animTimes = [60,60,60,60,60]
        self.movingRight = False
        self.movingLeft = False
        self.lookingRight = False
        self.lookingLeft = False
        self.jumping = False
        self.ducking = False
        self.falling = False
        self.dying = False
        self.jumpCounter = 0
        self.lives = 3
        self.points = 0
        self.keys = 0
        self.transporterCycle = 0
        self.blocksMax = 12
        self.blocks = self.blocksMax

        self.lookLeft()

    def moveRight(self):
        if (not self.movingRight) and (not self.dying):
            if self.ducking:
                self.animPtr = 4
            else:
                self.animPtr = 1
            self.animFrameMax = len(self.animList[self.animPtr]) - 1
            self.animFrame = 0
            self.movingRight = True
            self.lookingRight = False
            self.movingLeft = False
            self.lookingLeft = False

    def moveLeft(self):
        if (not self.movingLeft) and (not self.dying):
            if self.ducking:
                self.animPtr = 3
            else:
                self.animPtr = 0
            self.animFrameMax = len(self.animList[self.animPtr]) - 1
            self.animFrame = 0
            self.movingLeft = True
            self.lookingLeft = False
            self.movingRight = False
            self.lookingRight = False

    def lookRight(self):
        if (not self.lookingRight) and (not self.dying):
            if self.ducking:
                self.animPtr = 4
            else:
                self.animPtr = 1
            self.animFrame = 0
            self.animFrameMax = 0
            self.lookingRight = True
            self.movingRight = False
            self.lookingLeft = False
            self.movingLeft = False

    def lookLeft(self):
        if (not self.lookingLeft) and (not self.dying):
            if self.ducking:
                self.animPtr = 3
            else:
                self.animPtr = 0
            self.animFrame = 0
            self.animFrameMax = 0
            self.lookingLeft = True
            self.movingLeft = False
            self.lookingRight = False
            self.movingRight = False

    def jump(self):
        if (not self.jumping) and (not self.falling) and (not self.dying):
            if self.movingRight:
                self.animPtr = 1
                self.animFrameMax = len(self.animList[self.animPtr]) - 1
            elif self.lookingRight:
                self.animPtr = 1
                self.animFrameMax = 0
            elif self.movingLeft:
                self.animPtr = 0
                self.animFrameMax = len(self.animList[self.animPtr]) - 1
            elif self.lookingLeft:
                self.animPtr = 0
                self.animFrameMax = 0
            self.ducking = False
            self.jumping = True
            self.jumpCounter = 16
#            self.level.jumpSnd.play()

    def duck(self):
        if (not self.ducking) and (not self.jumping) and (not self.falling) and (not self.dying):
            br = self.getBR(7)
            bl = self.getBL(7)
            if (br == self.level.transporterNo) and (bl == self.level.transporterNo):
                self.teleport()
            else:
                if self.movingRight or self.lookingRight:
                    self.animPtr = 4
                elif self.movingLeft or self.lookingLeft:
                    self.animPtr = 3
                self.animFrame = 0
                self.animFrameMax = 0
                self.ducking = True

    def standUp(self):
        if self.ducking and (not self.dying):
            if self.movingRight:
                self.animPtr = 1
                self.animFrameMax = len(self.animList[self.animPtr]) - 1
            elif self.lookingRight:
                self.animPtr = 1
                self.animFrameMax = 0
            elif self.movingLeft:
                self.animPtr = 0
                self.animFrameMax = len(self.animList[self.animPtr]) - 1
            elif self.lookingLeft:
                self.animPtr = 0
                self.animFrameMax = 0
            self.animFrame = 0
            self.ducking = False

    def fall(self):
        if (not self.falling) and (not self.dying):
            self.falling = True
            self.jumping = False

    def move(self):
        if self.movingRight:
            if self.x < 448 - self.speed:
                bl = self.getBL(7)
                br = self.getBR(7)
                if (not self.falling) and (not self.jumping) and self.checkDown(7):
                    self.fall()
                elif (br == self.level.spikeNo) and (bl == self.level.spikeNo):
                    self.die()
                else:
                    rt = self.getRT(7)
                    rb = self.getRB(7)
                    lb = self.getLB(7)
                    if (rt == self.level.boxNo) and (rb == self.level.boxNo):
                        if (self.getRRT(7) == 0) and (self.getRRB(7) == 0):
                            x = (self.x / 32) + 1
                            self.moveBox(x, x + 1, self.y / 32)
                    elif (lb == self.level.trapNo) and (rb != self.level.trapNo):
                        self.level.frontData[self.y / 32][self.x / 32] = 2
                    elif (not self.checkObjLR(rt, rb, (self.x / 32) + 1, self.y / 32, self.lookRight)) and (not self.checkRight(7)):
                        self.x += self.speed
            else:
                self.x = 448
        elif self.movingLeft:
            if self.x > self.speed:
                bl = self.getBL(7)
                br = self.getBR(7)
                if (not self.falling) and (not self.jumping) and self.checkDown(7):
                    self.fall()
                elif (br == self.level.spikeNo) and (bl == self.level.spikeNo):
                    self.die()
                else:
                    lt = self.getLT(7)
                    lb = self.getLB(7)
                    rb = self.getRB(7)
                    if (lt == self.level.boxNo) and (lb == self.level.boxNo):
                        if (self.getLLT(7) == 0) and (self.getLLB(7) == 0):
                            x = self.x / 32
                            self.moveBox(x, x - 1, self.y / 32)
                    elif (rb == self.level.trapNo) and (lb != self.level.trapNo):
                        self.level.frontData[self.y / 32][(self.x / 32) + 1] = 2
                    elif (not self.checkObjLR(lt, lb, self.x / 32, self.y / 32, self.lookLeft)) and (not self.checkLeft(7)):
                        self.x -= self.speed
            else:
                self.x = 0
        if self.jumping:
            if self.y > self.speed:
                tl = self.getTL(8)
                tr = self.getTR(8)
                if (tl == 0) and (tr == 0) and (self.jumpCounter > 0):
                    self.y -= self.speed
                    self.jumpCounter -= 1
                elif not self.checkObjTB(tl, tr, (self.x + 8) / 32, (self.x + 24) / 32, (self.y - 1) / 32):
                    self.fall()
            else:
                self.y = 0
                self.fall()
        elif self.falling:
            if self.y < 448 - self.fallSpeed:
                bl = self.getBL(8)
                br = self.getBR(8)
                if (bl == 0) and (br == 0):
                    self.y += self.fallSpeed
                elif (br == self.level.spikeNo) and (bl == self.level.spikeNo):
                    self.die()
                elif not self.checkObjTB(bl, br, (self.x + 8) / 32, (self.x + 24) / 32, (self.y  + 32) / 32):
                    self.y = (self.y / 32) * 32 # sane y coordinate
                    self.falling = False
            else:
                self.y = 448
                self.falling = False
        if self.dying:
            if self.y < 448 - self.fallSpeed:
                self.y += self.fallSpeed
            else:
                self.lives -= 1
                if self.lives == 0:
                    self.screen.blit(self.level.gameOverGfx, (160,120))
                    pygame.display.flip()
                    while True:
                        evt = pygame.event.wait()
                        if evt.type == pygame.KEYDOWN:
                            break
                    self.level.quitGame = True
                else:
                    self.jumping = False
                    self.ducking = False
                    self.falling = False
                    self.dying = False
                    self.lookLeft()
                    self.level.load()
                    self.level.restart()

    def doAction(self):
        if self.movingRight or self.lookingRight:
            x = ((self.x + 22) / 32) + 1
            if self.ducking:
                y = (self.y / 32) + 1
            else:
                y = self.y / 32
            if x <= 14:
                self.setIceblock(x, y)
        elif self.movingLeft or self.lookingLeft:
            x = ((self.x + 9) / 32) - 1
            if self.ducking:
                y = (self.y / 32) + 1
            else:
                y = self.y / 32
            if x >= 0:
                self.setIceblock(x, y)

    def check(self):
        if not self.dying:
            for e in self.level.enemies:
                dx = self.x - e.x
                dy = self.y - e.y
                if (dx > -30) and (dx < 30) and (dy > -30) and (dy < 30):
                    self.die()

    def checkObjLR(self, pt, pb, x, y, look):
        if ((pt == self.level.switch1R) and (pb == self.level.switch1R)) or ((pt == self.level.switch1L) and (pb == self.level.switch1L)):
            self.level.doSwitch1()
            look()
        elif ((pt == self.level.switch2R) and (pb == self.level.switch2R)) or ((pt == self.level.switch2L) and (pb == self.level.switch2L)):
            self.level.doSwitch2()
            look()
        elif ((pt == self.level.switch3R) and (pb == self.level.switch3R)) or ((pt == self.level.switch3L) and (pb == self.level.switch3L)):
            self.level.doSwitch3()
            look()
        elif ((pt == self.level.switch4R) and (pb == self.level.switch4R)) or ((pt == self.level.switch4L) and (pb == self.level.switch4L)):
            self.level.doSwitch4()
            look()
        elif (pt == self.level.doorClosed) and (pb == self.level.doorClosed):
            self.openDoor(x, y)
        elif (pt == self.level.keyNo) and (pb == self.level.keyNo):
            self.newKey(x, y)
        elif (pt == self.level.heartNo) and (pb == self.level.heartNo):
            self.newLife(x, y)
        elif (pt == self.level.coin1) and (pb == self.level.coin1):
            self.incPoints(x, y, 10)
        elif (pt == self.level.coin2) and (pb == self.level.coin2):
            self.incPoints(x, y, 20)
        elif (pt == self.level.sapphire1) and (pb == self.level.sapphire1):
            self.incPoints(x, y, 30)
        elif (pt == self.level.sapphire2) and (pb == self.level.sapphire2):
            self.incPoints(x, y, 40)
        elif (pt == self.level.sapphire3) and (pb == self.level.sapphire3):
            self.incPoints(x, y, 50)
        elif (pt == self.level.sapphire4) and (pb == self.level.sapphire4):
            self.incPoints(x, y, 60)
        elif (pt == self.level.sapphire5) and (pb == self.level.sapphire5):
            self.incPoints(x, y, 70)
        elif (pt == self.level.sapphire6) and (pb == self.level.sapphire6):
            self.incPoints(x, y, 80)
        elif (pt == self.level.clockNo) and (pb == self.level.clockNo):
            self.getClock(x, y)
        else:
            return False
        return True

    def checkObjTB(self, pl, pr, xl, xr, y):
        if (pl == self.level.keyNo):
            self.newKey(xl, y)
        elif (pr == self.level.keyNo):
            self.newKey(xr, y)
        elif (pl == self.level.heartNo):
            self.newLife(xl, y)
        elif (pr == self.level.heartNo):
            self.newLife(xr, y)
        elif (pl == self.level.coin1):
            self.incPoints(xl, y, 10)
        elif (pr == self.level.coin1):
            self.incPoints(xr, y, 10)
        elif (pl == self.level.coin2):
            self.incPoints(xl, y, 20)
        elif (pr == self.level.coin2):
            self.incPoints(xr, y, 20)
        elif (pl == self.level.sapphire1):
            self.incPoints(xl, y, 30)
        elif (pr == self.level.sapphire1):
            self.incPoints(xr, y, 30)
        elif (pl == self.level.sapphire2):
            self.incPoints(xl, y, 40)
        elif (pr == self.level.sapphire2):
            self.incPoints(xr, y, 40)
        elif (pl == self.level.sapphire3):
            self.incPoints(xl, y, 50)
        elif (pr == self.level.sapphire3):
            self.incPoints(xr, y, 50)
        elif (pl == self.level.sapphire4):
            self.incPoints(xl, y, 60)
        elif (pr == self.level.sapphire4):
            self.incPoints(xr, y, 60)
        elif (pl == self.level.sapphire5):
            self.incPoints(xl, y, 70)
        elif (pr == self.level.sapphire5):
            self.incPoints(xr, y, 70)
        elif (pl == self.level.sapphire6):
            self.incPoints(xl, y, 80)
        elif (pr == self.level.sapphire6):
            self.incPoints(xr, y, 80)
        elif (pl == self.level.clockNo):
            self.getClock(xl, y)
        elif (pr == self.level.clockNo):
            self.getClock(xr, y)
        else:
            return False
        return True

    def die(self):
        if not self.dying:
            self.dying = True
            self.movingRight = False
            self.movingLeft = False
            self.lookingRight = False
            self.lookingLeft = False
            self.jumping = False
            self.ducking = False
            self.falling = False
            self.animPtr = 2
            self.animFrameMax = len(self.animList[self.animPtr]) - 1
            self.animFrame = 0
#            self.screen.fill(self.black, (480,70,160,32))
            self.keys = 0
            self.level.playerDieSnd.play()

    def setIceblock(self, x, y):
        t = self.level.frontData[y][x]
        if t == 0:
            if self.blocks > 0:
                self.blocks -= 1
                self.level.frontData[y][x] = 1
                self.screen.blit(self.level.frontGfx[1], (x*32,y*32))
                self.level.iceblockSnd.play()
                self.displayBlocks()
            else:
                self.level.noBlocksSnd.play()
        elif t == 1:
            if self.blocks < self.blocksMax:
                self.blocks += 1
                self.level.frontData[y][x] = 0
                self.screen.blit(self.level.backGfx[self.level.backData[y][x]], (x*32,y*32))
                self.level.iceblockSnd.play()
                self.displayBlocks()
            else:
                self.level.noBlocksSnd.play()

    def displayBlocks(self):
        x = 495
        y = 140
        self.screen.fill(self.black, (480, 140, 160, 160))
        for i in range(self.blocks):
            self.screen.blit(self.level.frontGfx[1], (x,y))
            x += 33
            if x >= 627:
                x = 495
                y += 33

    def moveBox(self, x1, x2, y):
        self.level.frontData[y][x1] = 0
        self.level.frontData[y][x2] = self.level.boxNo
        self.screen.blit(self.level.backGfx[self.level.backData[y][x1]], (x1*32,y*32))
        self.screen.blit(self.level.frontGfx[self.level.boxNo], (x2*32,y*32))
        self.level.boxSnd.play()

    def newLife(self, x, y):
        self.lives += 1
        self.level.frontData[y][x] = 0
        self.screen.blit(self.level.backGfx[self.level.backData[y][x]], (x*32,y*32))
        self.screen.fill(self.black, (490,30,150,20))
        self.level.font.write(490, 30, "LIVES:" + str(self.lives))
        self.level.heartSnd.play()

    def incPoints(self, x, y, p):
        self.level.frontData[y][x] = 0
        self.screen.blit(self.level.backGfx[self.level.backData[y][x]], (x*32,y*32))
        self.screen.fill(self.black, (490,50,150,20))
        self.points += p
        self.level.font.write(490, 50, "SCORE:" + str(self.points))
        self.level.pointSnd.play()

    def newKey(self, x, y):
        if self.keys < 4:
            self.keys += 1
            self.level.frontData[y][x] = 0
            self.screen.blit(self.level.backGfx[self.level.backData[y][x]], (x*32,y*32))
            self.screen.blit(self.level.frontGfx[self.level.keyNo], (470 + (self.keys*32), 70))
            self.level.keySnd.play()

    def openDoor(self, x, y):
        if self.keys > 0:
            self.screen.fill(self.black, (470 + (self.keys*32), 70, 32, 32))
            self.keys -= 1
            self.level.frontData[y][x] = self.level.doorOpened
            self.level.displayXY(x,y)
            self.level.doorSnd.play()

    def getClock(self, x, y):
        self.level.frontData[y][x] = 0
        self.level.clockSnd.play()
        for e in self.level.enemies:
            e.sleep()

    def teleport(self):
        (nx, ny) = self.level.transporters[self.transporterCycle]
        x = (self.x + 16) / 32
        y = self.y / 32
        while True:
            t = self.level.frontData[ny][nx]
            if ((t == 0) or (t >= self.level.keyNo)) and ((nx != x) or (ny != y)):
                break
            self.transporterCycle += 1
            if self.transporterCycle == len(self.level.transporters):
                self.transporterCycle = 0
            (nx, ny) = self.level.transporters[self.transporterCycle]

        self.restoreBG()
        self.x = nx * 32
        self.y = ny * 32
        self.transporterCycle += 1
        if self.transporterCycle == len(self.level.transporters):
            self.transporterCycle = 0
        self.level.transporterSnd.play()
