import pygame
import gfxobject

# By Willi Kappler <grandor@gmx.de>
# Licensed under GPL

class Skull(gfxobject.Object):
    def __init__(self, screen, level, gfx, x, y):
        gfxobject.Object.__init__(self, screen, level, gfx, x, y)

        self.animList = [[0,1,2,1],[3,4,5,4],[6,6]]
        self.moveLeft()
        self.moveAction = [self.turnLeft, self.turnRight, self.moveDown, self.moveUp]
        self.moveAnim = [1,0,2,2,2]
        self.animTimes = [60,60,200]

    def move(self):
        if self.asleep:
            if pygame.time.get_ticks() > self.sleepTime + 5000:
                self.asleep = False
        else:
            if self.direction == 1: # right
                if self.x < 448 - self.speed:
                    self.x += self.speed
                else:
                    self.x = 448
                    self.turnLeft()
            elif self.direction == 2: # left
                if self.x > self.speed:
                    self.x -= self.speed
                else:
                    self.x = 0
                    self.turnRight()
            elif self.direction == 5:
                if self.animFrame == self.animFrameMax:
                    self.moveRight()
            elif self.direction == 6:
                if self.animFrame == self.animFrameMax:
                    self.moveLeft()

    def turnRight(self):
        self.direction = 5
        self.animPtr = 2
        self.animFrame = 0
        self.animFrameMax = len(self.animList[2]) - 1

    def turnLeft(self):
        self.direction = 6
        self.animPtr = 2
        self.animFrame = 0
        self.animFrameMax = len(self.animList[2]) - 1

    def check(self):
        if self.direction == 1:
            rt = self.getRT()
            rb = self.getRB()
            if (rt == self.level.switch1R) and (rb == self.level.switch1R):
                self.turnLeft()
                self.level.doSwitch1()
            elif (rt == self.level.switch2R) and (rb == self.level.switch2R):
                self.turnLeft()
                self.level.doSwitch2()
            elif (rt == self.level.switch3R) and (rb == self.level.switch3R):
                self.turnLeft()
                self.level.doSwitch3()
            elif (rt == self.level.switch4R) and (rb == self.level.switch4R):
                self.turnLeft()
                self.level.doSwitch4()
            else:
                br = self.getBR()
                if self.checkRight() or (br == 0) or (br >= self.level.keyNo):
                    self.turnLeft()
        elif self.direction == 2:
            lt = self.getLT()
            lb = self.getLB()
            if (lt == self.level.switch1L) and (lb == self.level.switch1L):
                self.turnRight()
                self.level.doSwitch1()
            elif (lt == self.level.switch2L) and (lb == self.level.switch2L):
                self.turnRight()
                self.level.doSwitch2()
            elif (lt == self.level.switch3L) and (lb == self.level.switch3L):
                self.turnRight()
                self.level.doSwitch3()
            elif (lt == self.level.switch4L) and (lb == self.level.switch4L):
                self.turnRight()
                self.level.doSwitch4()
            else:
                bl = self.getBL()
                if self.checkLeft() or (bl == 0) or (bl >= self.level.keyNo):
                    self.turnRight()

