#!/usr/bin/env python

# Snowball V 0.1 - 2005.08.14
# By Willi Kappler <grandor@gmx.de>
#
# This game is licensed under the GPL
# See www.gnu.org for more information

import sys, pygame
sys.path.append("modules")
import editor
import font
import options
import highscore
import game
import level

pygame.init()

pygame.key.set_repeat(300, 30)
pygame.mouse.set_visible(0)

defaultOpt = options.Options()
defaultOpt.load()

screen = pygame.display.set_mode( (640,480), pygame.HWSURFACE | defaultOpt.mode )
defaultOpt.screen = screen

black = (0, 0, 0)

highsc = highscore.Highscore(screen)
highsc.load()

logoGfx = pygame.image.load("gfx/logo.png")
arrowLeftGfx = pygame.image.load("gfx/arrow_left.png")
arrowRightGfx = pygame.image.load("gfx/arrow_right.png")

defaultOpt.arrowRightGfx = arrowRightGfx

selectNextSnd = pygame.mixer.Sound("snd/tick.ogg")
selectSnd = pygame.mixer.Sound("snd/beeps2.ogg")

defaultOpt.selectNextSnd = selectNextSnd
defaultOpt.selectSnd = selectSnd

arrowLeftPos = [(320+118,238),(320+140,288),(320+118,338),(320+152,388),(320+58,438)]
arrowRightPos = [(290-118,238),(290-140,288),(290-118,338),(290-152,388),(290-58,438)]

bigFont = font.Font(screen, "gfx/big_font.png", 24, 32)
defaultOpt.font = bigFont
highsc.font = bigFont

smallFont = font.Font(screen, "gfx/small_font.png", 13, 16)
snowballLevel = level.Level(screen, smallFont, defaultOpt)
defaultOpt.level = snowballLevel
highsc.font2 = smallFont


levelEd = editor.LevelEditor(screen, smallFont, snowballLevel)
snowball = game.Game(screen, snowballLevel, defaultOpt, highsc)

selection = 0
quitGame = False

mainMenu = [(230, "PLAY GAME"), (280, "SET OPTIONS"), (330, "HIGHSCORE"), (380, "LEVEL EDITOR"), (430, "QUIT")]

clock = pygame.time.Clock()

while not quitGame:
    screen.fill(black)
    screen.blit(logoGfx, (154,32))
    screen.blit(arrowLeftGfx, arrowLeftPos[selection])
    screen.blit(arrowRightGfx, arrowRightPos[selection])

    for i in range(5):
        (y, txt) = mainMenu[i]
        if i == selection:
            bigFont.dance(320, y, txt, True)
        else:
            bigFont.write(320, y, txt, True)

    for event in pygame.event.get([pygame.QUIT, pygame.KEYDOWN]):
        if event.type == pygame.QUIT:
            quitGame = True
        elif event.type == pygame.KEYDOWN:
            if event.key == pygame.K_ESCAPE:
                quitGame = True
            elif event.key == pygame.K_UP:
                selection -= 1
                if selection == -1:
                    selection = 4
                selectNextSnd.play()
            elif event.key == pygame.K_DOWN:
                selection += 1
                if selection == 5:
                    selection = 0
                selectNextSnd.play()
            elif event.key == pygame.K_RETURN:
                selectSnd.play()
                if selection == 0:
                    snowball.run()
                elif selection == 1:
                    defaultOpt.run()
                elif selection == 2:
                    highsc.run()
                elif selection == 3:
                    levelEd.run()
                    pygame.mouse.set_visible(0)
                elif selection == 4:
                    quitGame = True

    clock.tick(30)
    pygame.display.flip()

defaultOpt.save()
highsc.save()
pygame.time.wait(200)

print "bye!"
sys.exit()
